==> unixODBC-1.8.tar.gz: unixODBC-1.8/ODBCConfig/classDrivers.cpp <==
/**************************************************
 * classDrivers
 *
 * see classDrivers.h for doco
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 31.JAN.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include "classDrivers.h"

#define Inherited classDriversData

classDrivers::classDrivers( QWidget* parent, const char* name )
	: Inherited( parent, name )
{
	lvwDrivers->setAllColumnsShowFocus( true );

	connect( pbAdd, SIGNAL(clicked()), this, SLOT(Add()) );
	connect( pbRemove, SIGNAL(clicked()), this, SLOT(Delete()) );
	connect( pbConfigure, SIGNAL(clicked()), this, SLOT(Edit()) );

	Load();
}


classDrivers::~classDrivers()
{
	if ( hIni != NULL )
		iniClose( hIni );
	
}


void classDrivers::Add()
{
	QString				qsError					= "";

	classProperties		*pProperties;
	HODBCINSTPROPERTY	hFirstProperty	= NULL;
	HODBCINSTPROPERTY	hCurProperty	= NULL;
	HODBCINSTPROPERTY	hLastProperty;
	char				szINI[FILENAME_MAX+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	// SET UP PROPERTIES LIST
	hFirstProperty 						= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	memset( hFirstProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hFirstProperty->nPromptType			= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hFirstProperty->pNext				= NULL;
    hFirstProperty->bRefresh			= 0;
    hFirstProperty->hDLL				= NULL;
    hFirstProperty->pWidget				= NULL;
    hFirstProperty->pszHelp				= NULL;
	hFirstProperty->aPromptData			= NULL;
	strncpy( hFirstProperty->szName, "Name", INI_MAX_PROPERTY_NAME );
	strcpy( hFirstProperty->szValue, "" );
	hLastProperty = hFirstProperty;

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "Description", INI_MAX_PROPERTY_NAME );
	strcpy( hFirstProperty->szValue, "" );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Driver", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Setup", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "FileUsage", INI_MAX_PROPERTY_NAME );
	strcpy( hLastProperty->szValue, "1" );


	// ALLOW USER TO EDIT
	pProperties = new classProperties( this, "Properties", hFirstProperty );
	pProperties->setCaption( "New Driver" );
	pProperties->resize( 200, 400 );
	if ( pProperties->exec() )
	{
		/* DELETE ENTIRE SECTION IF IT EXISTS (no entry given) */
		if ( SQLWritePrivateProfileString( hFirstProperty->szValue, NULL, NULL, szINI ) == FALSE )
		{
			delete pProperties;
			FreeProperties( &hFirstProperty );
			qsError.sprintf( "Could not write to (%s)", szINI );
			QMessageBox::information( this, "ODBC Config",  qsError );
			return;
		}

		/* ADD ENTRIES; SECTION CREATED ON FIRST CALL */
		for ( hCurProperty = hFirstProperty->pNext; hCurProperty != NULL; hCurProperty = hCurProperty->pNext )
		{
			SQLWritePrivateProfileString( hFirstProperty->szValue, hCurProperty->szName, hCurProperty->szValue, szINI );
		}
	}
	delete pProperties;
	FreeProperties( &hFirstProperty );

	// RELOAD (slow but safe)
	Load();
}

void classDrivers::Edit()
{
	QMessageBox::information(	this, "ODBC Config",  "Not implemented yet. Try the odbcinst command line tool" );
}

void classDrivers::Delete()
{
	QListViewItem		*pListViewItem;
	char 				szINI[FILENAME_MAX+1];
	char 				*pszName;
	QString				qsError;
	DWORD				nErrorCode;
	char				szErrorMsg[FILENAME_MAX+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	// GET SELECT DATA SOURCE NAME
    pListViewItem = lvwDrivers->currentItem();
	if ( pListViewItem )
	{
		pszName = (char *)pListViewItem->text( 0 ).ascii();
	}
	else
	{
		QMessageBox::information( this, "ODBC Config",  "Please select a Driver from the list first" );
		return;
	}

	// DELETE ENTIRE SECTION IF IT EXISTS (given NULL entry)
	if ( SQLWritePrivateProfileString( pszName, NULL, NULL, szINI ) == FALSE )
	{
		qsError.sprintf( "Could not write property list for (%s)", pszName );
		QMessageBox::information( this, "ODBC Config",  qsError );
		while ( SQLInstallerError( 1, &nErrorCode, szErrorMsg, FILENAME_MAX, NULL ) == SQL_SUCCESS )
			QMessageBox::information( this, "ODBC Config",  szErrorMsg );
	}
	
	// RELOAD (slow but safe)
	Load();
}



void classDrivers::Load()
{
    lvwDrivers->clear();

	/***************************************
	 * METHOD 1 - Using Driver Manager
	 ***************************************/
	
			
	/***************************************
	 * METHOD 2 - Using ODBCINST lib (also see SQLGetInstalledDrivers)
	 ***************************************/
	/*
	QListViewItem	*pListViewItem;
	QString			qsError;
	DWORD			nErrorCode;
	char   			szErrorMsg[101];

	char			szINI[FILENAME_MAX+1];
	char			szSectionNames[4096];
	char			szSectionName[INI_MAX_OBJECT_NAME+1];
	int				nElement;
	
	char 			szDriverName[INI_MAX_OBJECT_NAME+1];
	char 			szDescription[INI_MAX_PROPERTY_VALUE+1];
	char 			szDriver[INI_MAX_PROPERTY_VALUE+1];
	char 			szSetup[INI_MAX_PROPERTY_VALUE+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	memset( szSectionNames, 0, sizeof(szSectionNames) );
	if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szSectionNames, 4090, szINI ) >= 0 )
	{
		for ( nElement = 0; iniElement( szSectionNames, '\0', '\0', nElement, szSectionName, INI_MAX_OBJECT_NAME ) == INI_SUCCESS ; nElement++ )
		{
			szDriverName[0]		= '\0';
			szDescription[0]	= '\0';
			szDriver[0]			= '\0';
			szSetup[0]			= '\0';
			SQLGetPrivateProfileString( szSectionName, "Driver", "", szDriverName, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Description", "", szDescription, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Driver", "", szDriver, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Setup", "", szSetup, INI_MAX_PROPERTY_VALUE, szINI );
			pListViewItem = new QListViewItem( lvwDrivers, szDriverName, szDescription, szDriver, szSetup );
		}
	}
	else
	{
		qsError.sprintf( "Could not load %s", szINI );
		QMessageBox::information(	this, "ODBC Config",  qsError );
		while ( SQLInstallerError( 1, &nErrorCode, szErrorMsg, 100, NULL ) == SQL_SUCCESS )
			QMessageBox::information( this, "ODBC Config",  szErrorMsg );
	}
	*/

	/***************************************
	 * METHOD 3 - Using INI lib
	 ***************************************/
	char 			szDriverName[INI_MAX_OBJECT_NAME+1];
	char			szPropertyName[INI_MAX_PROPERTY_NAME+1];
	char 			szDescription[INI_MAX_PROPERTY_VALUE+1];
	char 			szDriver[INI_MAX_PROPERTY_VALUE+1];
	char 			szSetup[INI_MAX_PROPERTY_VALUE+1];
	QListViewItem	*pListViewItem;
	QString			qsError;

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
	{
		iniObjectFirst( hIni );
		while ( iniObjectEOL( hIni ) == FALSE )
		{
			szDriverName[0] = '\0';
			szDescription[0] 	= '\0';
			szDriver[0] 	= '\0';
			szSetup[0] 		= '\0';
			iniObject( hIni, szDriverName );
			iniPropertyFirst( hIni );
            if ( strcmp( szDriverName, "ODBC" ) == 0 )
            {
                iniObjectNext( hIni );
                continue;
            }

			while ( iniPropertyEOL( hIni ) == FALSE )
			{
				iniProperty( hIni, szPropertyName );
				iniToUpper( szPropertyName );

				if ( strncmp( szPropertyName, "DESCRIPTION", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szDescription );
				if ( strncmp( szPropertyName, "DRIVER", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szDriver );
				if ( strncmp( szPropertyName, "SETUP", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szSetup );

				iniPropertyNext( hIni );
			}

			pListViewItem = new QListViewItem( lvwDrivers, szDriverName, szDescription, szDriver, szSetup );
			iniObjectNext( hIni );
		}
	}
	else
	{
		qsError.sprintf( "Could not open System odbc.ini file at %s", szINI );
		QMessageBox::information(	this, "ODBC Config",  qsError );
	}
	
}

void classDrivers::FreeProperties( HODBCINSTPROPERTY *hFirstProperty )
{
	HODBCINSTPROPERTY	hNextProperty;
	HODBCINSTPROPERTY	hCurProperty;

	/* SANITY CHECKS */
	if ( (*hFirstProperty) == NULL )
		return;

	/* FREE MEMORY */
	for ( hCurProperty = (*hFirstProperty); hCurProperty != NULL; hCurProperty = hNextProperty )
	{
		hNextProperty = hCurProperty->pNext;

		/* FREE ANY PROMPT DATA (ie pick list options and such) */
		if ( hCurProperty->aPromptData != NULL )
			free( hCurProperty->aPromptData );

		/* FREE OTHER STUFF */
        if ( hCurProperty->pszHelp != NULL )
			free( hCurProperty->pszHelp );

		free( hCurProperty );
	}
    (*hFirstProperty) = NULL;
}

==> unixODBC-1.8.3.tar.gz: unixODBC-1.8.3/ODBCConfig/classDrivers.cpp <==
/**************************************************
 * classDrivers
 *
 * see classDrivers.h for doco
 *
 **************************************************
 * This code was created by Peter Harvey @ CodeByDesign.
 * Released under GPL 31.JAN.99
 *
 * Contributions from...
 * -----------------------------------------------
 * Peter Harvey		- pharvey@codebydesign.com
 **************************************************/
#include "classDrivers.h"

#define Inherited classDriversData

classDrivers::classDrivers( QWidget* parent, const char* name )
	: Inherited( parent, name )
{
	lvwDrivers->setAllColumnsShowFocus( true );

	connect( pbAdd, SIGNAL(clicked()), this, SLOT(Add()) );
	connect( pbRemove, SIGNAL(clicked()), this, SLOT(Delete()) );
	connect( pbConfigure, SIGNAL(clicked()), this, SLOT(Edit()) );

	Load();
}


classDrivers::~classDrivers()
{
	if ( hIni != NULL )
		iniClose( hIni );
	
}


void classDrivers::Add()
{
	QString				qsError					= "";

	classProperties		*pProperties;
	HODBCINSTPROPERTY	hFirstProperty	= NULL;
	HODBCINSTPROPERTY	hCurProperty	= NULL;
	HODBCINSTPROPERTY	hLastProperty;
	char				szINI[FILENAME_MAX+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	// SET UP PROPERTIES LIST
	hFirstProperty 						= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	memset( hFirstProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hFirstProperty->nPromptType			= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hFirstProperty->pNext				= NULL;
    hFirstProperty->bRefresh			= 0;
    hFirstProperty->hDLL				= NULL;
    hFirstProperty->pWidget				= NULL;
    hFirstProperty->pszHelp				= NULL;
	hFirstProperty->aPromptData			= NULL;
	strncpy( hFirstProperty->szName, "Name", INI_MAX_PROPERTY_NAME );
	strcpy( hFirstProperty->szValue, "" );
	hLastProperty = hFirstProperty;

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "Description", INI_MAX_PROPERTY_NAME );
	strcpy( hLastProperty->szValue, "" );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Driver", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Setup", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "FileUsage", INI_MAX_PROPERTY_NAME );
	strcpy( hLastProperty->szValue, "1" );


	// ALLOW USER TO EDIT
	pProperties = new classProperties( this, "Properties", hFirstProperty );
	pProperties->setCaption( "New Driver" );
	pProperties->resize( 200, 400 );
	if ( pProperties->exec() )
	{
		/* DELETE ENTIRE SECTION IF IT EXISTS (no entry given) */
		if ( SQLWritePrivateProfileString( hFirstProperty->szValue, NULL, NULL, szINI ) == FALSE )
		{
			delete pProperties;
			FreeProperties( &hFirstProperty );
			qsError.sprintf( "Could not write to (%s)", szINI );
			QMessageBox::information( this, "ODBC Config",  qsError );
			return;
		}

		/* ADD ENTRIES; SECTION CREATED ON FIRST CALL */
		for ( hCurProperty = hFirstProperty->pNext; hCurProperty != NULL; hCurProperty = hCurProperty->pNext )
		{
			SQLWritePrivateProfileString( hFirstProperty->szValue, hCurProperty->szName, hCurProperty->szValue, szINI );
		}
	}
	delete pProperties;
	FreeProperties( &hFirstProperty );

	// RELOAD (slow but safe)
	Load();
}

void classDrivers::Edit()
{
	QString				qsName					= "";
	QString				qsError					= "";

	classProperties		*pProperties;
	HODBCINSTPROPERTY	hFirstProperty	= NULL;
	HODBCINSTPROPERTY	hCurProperty	= NULL;
	HODBCINSTPROPERTY	hLastProperty;
	char				szINI[FILENAME_MAX+1];
	QListViewItem		*pListViewItem;

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	// HAS THE USER SELECTED SOMETHING
    pListViewItem = lvwDrivers->currentItem();
	if ( pListViewItem )
		qsName		= pListViewItem->text( 0 );
	else
	{
		QMessageBox::information( this, "ODBC Config",  "Please select a Driver from the list first." );
		return;
	}


	// SET UP PROPERTIES LIST
	hFirstProperty 						= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	memset( hFirstProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hFirstProperty->nPromptType			= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hFirstProperty->pNext				= NULL;
    hFirstProperty->bRefresh			= 0;
    hFirstProperty->hDLL				= NULL;
    hFirstProperty->pWidget				= NULL;
    hFirstProperty->pszHelp				= NULL;
	hFirstProperty->aPromptData			= NULL;
	strncpy( hFirstProperty->szName, "Name", INI_MAX_PROPERTY_NAME );
	strcpy( hFirstProperty->szValue, qsName.data() );
	hLastProperty = hFirstProperty;

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "Description", INI_MAX_PROPERTY_NAME );
	strcpy( hLastProperty->szValue, "" );
	SQLGetPrivateProfileString( qsName.data(), hLastProperty->szName, "", hLastProperty->szValue, sizeof(hLastProperty->szValue)-1, szINI );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Driver", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
	SQLGetPrivateProfileString( qsName.data(), hLastProperty->szName, "", hLastProperty->szValue, sizeof(hLastProperty->szValue)-1, szINI );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType			= ODBCINST_PROMPTTYPE_FILENAME;
	strncpy( hLastProperty->szName, "Setup", INI_MAX_PROPERTY_NAME );
	strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
	SQLGetPrivateProfileString( qsName.data(), hLastProperty->szName, "", hLastProperty->szValue, sizeof(hLastProperty->szValue)-1, szINI );

	hLastProperty->pNext 				= (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
	hLastProperty 						= hLastProperty->pNext;
	memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
	hLastProperty->nPromptType				= ODBCINST_PROMPTTYPE_TEXTEDIT;
	hLastProperty->pNext					= NULL;
    hLastProperty->bRefresh					= 0;
    hLastProperty->hDLL						= NULL;
    hLastProperty->pWidget					= NULL;
    hFirstProperty->pszHelp					= NULL;
	hFirstProperty->aPromptData				= NULL;
	strncpy( hLastProperty->szName, "FileUsage", INI_MAX_PROPERTY_NAME );
	strcpy( hLastProperty->szValue, "1" );
	SQLGetPrivateProfileString( qsName.data(), hLastProperty->szName, "", hLastProperty->szValue, sizeof(hLastProperty->szValue)-1, szINI );


	// ALLOW USER TO EDIT
	pProperties = new classProperties( this, "Properties", hFirstProperty );
	pProperties->setCaption( "New Driver" );
	pProperties->resize( 200, 400 );
	if ( pProperties->exec() )
	{
		/* DELETE ENTIRE SECTION IF IT EXISTS (no entry given) */
		if ( SQLWritePrivateProfileString( qsName.ascii(), NULL, NULL, szINI ) == FALSE )
		{
			delete pProperties;
			FreeProperties( &hFirstProperty );
			qsError.sprintf( "Could not write to (%s)", szINI );
			QMessageBox::information( this, "ODBC Config",  qsError );
			return;
		}

		/* ADD ENTRIES; SECTION CREATED ON FIRST CALL */
		for ( hCurProperty = hFirstProperty->pNext; hCurProperty != NULL; hCurProperty = hCurProperty->pNext )
		{
			SQLWritePrivateProfileString( hFirstProperty->szValue, hCurProperty->szName, hCurProperty->szValue, szINI );
		}
	}
	delete pProperties;
	FreeProperties( &hFirstProperty );

	// RELOAD (slow but safe)
	Load();
}

void classDrivers::Delete()
{
	QListViewItem		*pListViewItem;
	char 				szINI[FILENAME_MAX+1];
	char 				*pszName;
	QString				qsError;
	DWORD				nErrorCode;
	char				szErrorMsg[FILENAME_MAX+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	// GET SELECT DATA SOURCE NAME
    pListViewItem = lvwDrivers->currentItem();
	if ( pListViewItem )
	{
		pszName = (char *)pListViewItem->text( 0 ).ascii();
	}
	else
	{
		QMessageBox::information( this, "ODBC Config",  "Please select a Driver from the list first" );
		return;
	}

	// DELETE ENTIRE SECTION IF IT EXISTS (given NULL entry)
	if ( SQLWritePrivateProfileString( pszName, NULL, NULL, szINI ) == FALSE )
	{
		qsError.sprintf( "Could not write property list for (%s)", pszName );
		QMessageBox::information( this, "ODBC Config",  qsError );
		while ( SQLInstallerError( 1, &nErrorCode, szErrorMsg, FILENAME_MAX, NULL ) == SQL_SUCCESS )
			QMessageBox::information( this, "ODBC Config",  szErrorMsg );
	}
	
	// RELOAD (slow but safe)
	Load();
}



void classDrivers::Load()
{
    lvwDrivers->clear();

	/***************************************
	 * METHOD 1 - Using Driver Manager
	 ***************************************/
	
			
	/***************************************
	 * METHOD 2 - Using ODBCINST lib (also see SQLGetInstalledDrivers)
	 ***************************************/
	/*
	QListViewItem	*pListViewItem;
	QString			qsError;
	DWORD			nErrorCode;
	char   			szErrorMsg[101];

	char			szINI[FILENAME_MAX+1];
	char			szSectionNames[4096];
	char			szSectionName[INI_MAX_OBJECT_NAME+1];
	int				nElement;
	
	char 			szDriverName[INI_MAX_OBJECT_NAME+1];
	char 			szDescription[INI_MAX_PROPERTY_VALUE+1];
	char 			szDriver[INI_MAX_PROPERTY_VALUE+1];
	char 			szSetup[INI_MAX_PROPERTY_VALUE+1];

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	memset( szSectionNames, 0, sizeof(szSectionNames) );
	if ( SQLGetPrivateProfileString( NULL, NULL, NULL, szSectionNames, 4090, szINI ) >= 0 )
	{
		for ( nElement = 0; iniElement( szSectionNames, '\0', '\0', nElement, szSectionName, INI_MAX_OBJECT_NAME ) == INI_SUCCESS ; nElement++ )
		{
			szDriverName[0]		= '\0';
			szDescription[0]	= '\0';
			szDriver[0]			= '\0';
			szSetup[0]			= '\0';
			SQLGetPrivateProfileString( szSectionName, "Driver", "", szDriverName, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Description", "", szDescription, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Driver", "", szDriver, INI_MAX_PROPERTY_VALUE, szINI );
			SQLGetPrivateProfileString( szSectionName, "Setup", "", szSetup, INI_MAX_PROPERTY_VALUE, szINI );
			pListViewItem = new QListViewItem( lvwDrivers, szDriverName, szDescription, szDriver, szSetup );
		}
	}
	else
	{
		qsError.sprintf( "Could not load %s", szINI );
		QMessageBox::information(	this, "ODBC Config",  qsError );
		while ( SQLInstallerError( 1, &nErrorCode, szErrorMsg, 100, NULL ) == SQL_SUCCESS )
			QMessageBox::information( this, "ODBC Config",  szErrorMsg );
	}
	*/

	/***************************************
	 * METHOD 3 - Using INI lib
	 ***************************************/
	char 			szDriverName[INI_MAX_OBJECT_NAME+1];
	char			szPropertyName[INI_MAX_PROPERTY_NAME+1];
	char 			szDescription[INI_MAX_PROPERTY_VALUE+1];
	char 			szDriver[INI_MAX_PROPERTY_VALUE+1];
	char 			szSetup[INI_MAX_PROPERTY_VALUE+1];
	QListViewItem	*pListViewItem;
	QString			qsError;

#ifdef SYSTEM_FILE_PATH
    sprintf( szINI, "%s/odbcinst.ini", SYSTEM_FILE_PATH );
#else
    strcpy( szINI, "/etc/odbcinst.ini" );
#endif

	if ( iniOpen( &hIni, szINI, '#', '[', ']', '=', TRUE ) != INI_ERROR )
	{
		iniObjectFirst( hIni );
		while ( iniObjectEOL( hIni ) == FALSE )
		{
			szDriverName[0] = '\0';
			szDescription[0] 	= '\0';
			szDriver[0] 	= '\0';
			szSetup[0] 		= '\0';
			iniObject( hIni, szDriverName );
			iniPropertyFirst( hIni );
            if ( strcmp( szDriverName, "ODBC" ) == 0 )
            {
                iniObjectNext( hIni );
                continue;
            }

			while ( iniPropertyEOL( hIni ) == FALSE )
			{
				iniProperty( hIni, szPropertyName );
				iniToUpper( szPropertyName );

				if ( strncmp( szPropertyName, "DESCRIPTION", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szDescription );
				if ( strncmp( szPropertyName, "DRIVER", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szDriver );
				if ( strncmp( szPropertyName, "SETUP", INI_MAX_PROPERTY_NAME ) == 0 )
					iniValue( hIni, szSetup );

				iniPropertyNext( hIni );
			}

			pListViewItem = new QListViewItem( lvwDrivers, szDriverName, szDescription, szDriver, szSetup );
			iniObjectNext( hIni );
		}
	}
	else
	{
		qsError.sprintf( "Could not open System odbc.ini file at %s", szINI );
		QMessageBox::information(	this, "ODBC Config",  qsError );
	}
	
}

void classDrivers::FreeProperties( HODBCINSTPROPERTY *hFirstProperty )
{
	HODBCINSTPROPERTY	hNextProperty;
	HODBCINSTPROPERTY	hCurProperty;

	/* SANITY CHECKS */
	if ( (*hFirstProperty) == NULL )
		return;

	/* FREE MEMORY */
	for ( hCurProperty = (*hFirstProperty); hCurProperty != NULL; hCurProperty = hNextProperty )
	{
		hNextProperty = hCurProperty->pNext;

		/* FREE ANY PROMPT DATA (ie pick list options and such) */
		if ( hCurProperty->aPromptData != NULL )
			free( hCurProperty->aPromptData );

		/* FREE OTHER STUFF */
        if ( hCurProperty->pszHelp != NULL )
			free( hCurProperty->pszHelp );

		free( hCurProperty );
	}
    (*hFirstProperty) = NULL;
}
